'
' MacrCase.bas - for PB/WIN 7.0 or later..
' Public Domain by Borje Hagsten, April 2003
'--------------------------------------------------------------------
' Some powerful asm L/M/T/UCASE MACRO functions that calls subs which
' uses localized character tables via global string tables, initiated
' by CharUpper and CharLower functions. This, IMHO, is how PB's U/M/LCASE
' should work, but currently do not (they only treat English alphabeth..)
' Also included an mROT13 routine that does very fast ROT13 conversion.
' You can extract what you need, or place them all in a DLL, whatever..
'
' LOG:
' Apr 8: Rewrote AnsiMCASE and AnsiTCASE, so all routines now use locale
'        ANSI tables. Both those routines thereby also became a bit faster..
'        Moved LEN test to subs, to ensure their safety if used alone.
'        Made some minor trimming of ROT13 routine and edited comments. 
'
' Note, these macros/subs can also, with minor tweaking of initAnsiTables
' routine, be used for things like encrytion/decrytion, etc.
' Also note - MACRO's become inline, so compiled files grows with
' the number of uses in code. With speed and usefulness comes a small
' extra burdon in form of a few extra bytes, but at least I can live
' with that. See also PB help for more info on MACRO.
'
' Also-also note - each MACRO FUNCTION uses a SUB that alters given
' string in-place for best speed. These subs can of course also be
' used directly, if given string don't have to be preserved. If the
' SUB's are used directly, like "AnsiUCASE txt", you get optimal speed..
'
' Credits to Semen Matusovski, Steve Hutchesson and Lance Edmunds
' for your always so interesting contributions in the PB Forums.
' Parts of this code originates from your stuff, fellows - thank you!
'
' This code and compiled test prog can also be downloaded from my
' PB web-page at  http://www.tolkenxp.com/pb
'--------------------------------------------------------------------
 
#COMPILE EXE
#INCLUDE "WIN32API.INC"
'--------------------------------------------------------------------
%ID_UPPER   = 110
%ID_LOWER   = 111
%ID_TCASE   = 112
%ID_MCASE   = 113
%ID_ROT13   = 114
%ID_PBUCASE = 115
%ID_PBLCASE = 116
%ID_PBMCASE = 117
%ID_TEXT    = 130
%ID_TEXT2   = 132
%ID_CHECK   = 140
$TXT        = "Here's some text: verstt   "
'--------------------------------------------------------------------
DECLARE CALLBACK FUNCTION DlgProc() AS LONG
DECLARE SUB AnsiLCASE (a$)
DECLARE SUB AnsiMCASE (a$)
DECLARE SUB AnsiTCASE (a$)
DECLARE SUB AnsiUCASE (a$)
DECLARE SUB asmROT13  (a$)
DECLARE SUB CaseProc  (BYVAL hDlg AS DWORD, BYVAL cmd AS LONG)
DECLARE SUB initAnsiTables
'--------------------------------------------------------------------
GLOBAL gLCaseTable AS STRING, gMCaseTable AS STRING, _ ' these need to be global..
       gTCaseTable AS STRING, gUCaseTable AS STRING
 
'
' returns LCASE string without altering given string
' use like: txt = mLCASER(txt2)
'--------------------------------------------------------------------
MACRO FUNCTION mLCASER(txtIn)
  MACROTEMP txtOut
  LOCAL txtOut AS STRING
  txtOut = txtIn              ' make a copy of string to convert
  AnsiLCASE txtOut            ' call separate AnsiLCASE routine
END MACRO = txtOut            ' return converted string
 
'
' returns MCASE string without altering given string
' use like: txt = mMCASER(txt2)
'--------------------------------------------------------------------
MACRO FUNCTION mMCASER(txtIn)
  MACROTEMP txtOut
  LOCAL txtOut AS STRING
  txtOut = txtIn              ' make a copy of string to convert
  AnsiMCASE txtOut            ' call separate AnsiMCASE routine
END MACRO = txtOut            ' return converted string
 
'
' returns TCASE string with toggled case, without altering given string
' use like: txt = mTCASER(txt2)
'--------------------------------------------------------------------
MACRO FUNCTION mTCASER(txtIn)
  MACROTEMP txtOut
  LOCAL txtOut AS STRING
  txtOut = txtIn              ' make a copy of string to convert
  AnsiTCASE txtOut            ' call separate AnsiTCASE routine
END MACRO = txtOut            ' return converted string
 
'
' returns UCASE string without altering given string
' use like: txt = mUCASER(txt2)
'--------------------------------------------------------------------
MACRO FUNCTION mUCASER(txtIn)
  MACROTEMP txtOut
  LOCAL txtOut AS STRING
  txtOut = txtIn              ' make a copy of string to convert
  AnsiUCASE txtOut            ' call separate AnsiUCASE routine
END MACRO = txtOut            ' return converted string
 
'
' returns ROT13 converted string without altering given string
' use like: txt = mROT13(txt2)
'--------------------------------------------------------------------
MACRO FUNCTION mROT13(txtIn)
  MACROTEMP txtOut
  LOCAL txtOut AS STRING
  txtOut = txtIn              ' make a copy of string to convert
  asmROT13 txtOut             ' call separate asmROT13 routine
END MACRO = txtOut            ' return converted string
 
'
' initializes the ANSI character look-up tables for case conversion
' CharUpper/CharLower uses default locale (Win95/98) or current
' language driver (NT), so tables become "localized".
' Note: this is one use for this approach. Another way could be by
' using specially designed tables for encryption/decryption purposes..
'--------------------------------------------------------------------
SUB initAnsiTables
  LOCAL c AS LONG, pStr AS BYTE PTR
  gLCaseTable = CHR$(1, 1 TO 255)     ' first byte in string must not be zero
  gTCaseTable = CHR$(1, 1 TO 255)     ' (we don't need first byte anyway, so..)
  gUCaseTable = CHR$(1, 1 TO 255)     ' 
  gMCaseTable = STRING$(256, 1)       ' for compare, fill this one with 1's
 
  CharUpper BYVAL STRPTR(gUCaseTable) ' create a table for UCASE conversion
  CharLower BYVAL STRPTR(gLCaseTable) ' create a table for LCASE conversion
 
  pStr = STRPTR(gTCaseTable)   ' create a table for tOOGLED case conversion
  FOR c = 1 TO 255
     IF IsCharAlpha(c) THEN
        IF IsCharUpper(c) THEN
           CharLower BYVAL pStr
        ELSEIF IsCharLower(c) THEN
           CharUpper BYVAL pStr
        END IF
     END IF
     INCR pStr
  NEXT  
 
  ' create a ANSI look-up table for Mixed case compare
  ' works like this: If ASCII number is a char, set byte to 2, else it'll be 1.
  ' This mimics standard Mixed case behavour, like in PB's MCASE$. A problem
  ' with that is for example difference between words and names with apostroph
  ' in the (non-character), like "where's" and "O'Connery". Using standard way,
  ' "where's" becomes "where'S", but "O'Connery" is correct. If we also would
  ' include ' (39) as character, "where's" would become correct, but "O'connery"
  ' would look wrong. If you don't care about "O'connery", add "OR c = 39" to
  ' IsCharAlpha line. And/or whatever other characters you like, like digits?
  ' Else leave it be..
  pStr = STRPTR(gMCaseTable)
  INCR pStr
  FOR c = 1 TO 255
     IF IsCharAlpha(c) THEN @pStr = 2
     INCR pStr
  NEXT  
 
END SUB
 
'
' Lower case routine, using global look-up table
' Note - given string is converted "in-place" for best speed
'--------------------------------------------------------------------
SUB AnsiLCASE (a$)
  IF LEN(a$) = 0 THEN EXIT SUB
  #REGISTER NONE
  LOCAL pTbl AS DWORD
  pTbl = STRPTR(gLCaseTable)  ' pointer to ansi LCASE character table
 
  ! mov esi, a$               ; eax = pointer to string handle
  ! mov esi, [esi]            ; eax = pointer to string data
  ! mov ecx, [esi-4]          ; move length of string into ecx
  ! mov ebx, pTbl             ; move pointer to translation table into ebx
 
AnsiLCASE_Loop:
  ! mov al, [esi]             ; move current character into al
  ! cmp al, 0                 ; look out for for zero bytes
  ! je AnsiLCASE_Next         ; if zero byte, jump past character translation
  ! xlatb                     ; else translate character
 
AnsiLCASE_Next:
  ! mov [esi], al             ; move translated character back into string
  ! inc esi                   ; next character
  ! dec ecx                   ; decr counter
  ! jnz AnsiLCASE_Loop        ; exit when ecx = 0
 
END SUB
 
'
' Mixed case routine, using global look-up tables
' Note - given string is converted "in-place" for best speed
'--------------------------------------------------------------------
SUB AnsiMCASE (a$)
  IF LEN(a$) = 0 THEN EXIT SUB
  #REGISTER NONE
  LOCAL pTbl AS DWORD, pUTbl AS DWORD, pLTbl AS DWORD, wordflag AS BYTE
  pTbl  = STRPTR(gMCaseTable)  ' pointer to ansi MCASE compare table
  pUTbl = STRPTR(gUCaseTable)  ' pointer to ansi UCASE translation table
  pLTbl = STRPTR(gLCaseTable)  ' pointer to ansi LCASE translation table
 
  ! mov esi, a$               ; eax = pointer to string handle
  ! mov esi, [esi]            ; eax = pointer to string data
  ! mov ecx, [esi-4]          ; move length of string into ecx
 
AnsiMCASE_Loop:
  ! mov ebx, pTbl             ; move pointer to compare table into ebx
  ! mov al, [esi]             ; move current character in al
  ! mov dl, al                ; also store character in dl
  ! xlatb                     ; translate character in al
  ! cmp al, 1                 ; see if was a character or not
  ! je AnsiMCASE_ReSetFlag    ; if not, reset wordflag
  ! cmp wordFlag, 0           ; else a character, compare with wordflag
  ! je AnsiMCASE_Upper        ; if wordflag was zero it's first letter, so convert to ucase
  ! jmp AnsiMCASE_Lower       ; else make lcase (rest of the word)
 
AnsiMCASE_ReSetFlag:
  ! mov wordFlag, 0           ; reset word flag
  ! mov al, dl                ; move stored character back into al
  ! jmp AnsiMCASE_Next        ; move on to next character
 
AnsiMCASE_Upper:
  ! mov al, dl                ; move original character back into al
  ! mov ebx, pUTbl            ; move pointer to upper case table into ebx
  ! xlatb                     ; translate character to UCASE
  ! mov [esi], al             ; move translated character back into string
  ! mov wordFlag, 1           ; we're in a word, so make sure word flag is set
  ! jmp AnsiMCASE_Next        ; move on to next character
 
AnsiMCASE_Lower:
  ! mov al, dl                ; move original character back into al
  ! mov ebx, pLTbl            ; move pointer to lower case table into ebx
  ! xlatb                     ; translate character LCASE
  ! mov [esi], al             ; move translated character back into string
  ! mov wordFlag, 1           ; we're in a word, so make sure word flag is set
 
AnsiMCASE_Next:
  ! inc esi                   ; next character
  ! dec ecx                   ; decr counter
  ! jnz AnsiMCASE_Loop        ; exit when ecx = 0
 
END SUB
 
'
' Toggle case routine - toggles UPPER to lower and lower to UPPER..
' Note - given string is converted "in-place" for best speed
'--------------------------------------------------------------------
SUB AnsiTCASE(a$)
  IF LEN(a$) = 0 THEN EXIT SUB
  #REGISTER NONE
  LOCAL pTbl AS DWORD
  pTbl = STRPTR(gTCaseTable)  ' pointer to ansi TCASE character table
 
  ! mov esi, a$               ; eax = pointer to string handle
  ! mov esi, [esi]            ; eax = pointer to string data
  ! mov ecx, [esi-4]          ; move length of string into ecx
  ! mov ebx, pTbl             ; move pointer to translation table into ebx
 
AnsiTCASE_Loop:
  ! mov al, [esi]             ; move current character into al
  ! cmp al, 0                 ; look out for for zero bytes
  ! je AnsiTCASE_Next         ; if zero byte, jump past character translation
  ! xlatb                     ; else translate character
 
AnsiTCASE_Next:
  ! mov [esi], al             ; move translated character back into string
  ! inc esi                   ; next character
  ! dec ecx                   ; decr counter
  ! jnz AnsiTCASE_Loop        ; exit when ecx = 0
 
END SUB
 
'
' Upper case routine, using global look-up table
' Note - given string is converted "in-place" for best speed
'--------------------------------------------------------------------
SUB AnsiUCASE (a$)
  IF LEN(a$) = 0 THEN EXIT SUB
  #REGISTER NONE
  LOCAL pTbl AS DWORD
  pTbl = STRPTR(gUCaseTable)  ' pointer to ansi UCASE character table
 
  ! mov esi, a$               ; eax = pointer to string handle
  ! mov esi, [esi]            ; eax = pointer to string data
  ! mov ecx, [esi-4]          ; move length of string into ecx
  ! mov ebx, pTbl             ; move pointer to translation table into ebx
 
AnsiUCASE_Loop:
  ! mov al, [esi]             ; move current character into al
  ! cmp al, 0                 ; look out for zero bytes
  ! je AnsiUCASE_Next         ; if zero byte, jump past character translation
  ! xlatb                     ; else translate character
 
AnsiUCASE_Next:
  ! mov [esi], al             ; move translated character back into string
  ! inc esi                   ; next character
  ! dec ecx                   ; decr counter
  ! jnz AnsiUCASE_Loop        ; exit when ecx = 0
 
END SUB
 
'
' asmROT13 encryption/decryption - no extended ANSI here (a-z, A-Z only).
' Repeat action to encrypt/decrypt
'------------------------------------------------------------------------
SUB asmROT13(a$)
  IF LEN(a$) = 0 THEN EXIT SUB
  #REGISTER NONE
 
  ! mov esi, a$            ; esi = pointer to string handle
  ! mov esi, [esi]         ; esi = pointer to string data
  ! mov ecx, [esi-4]       ; length of string
 
  BgnRot13Loop:
     ! mov al, [esi]       ; move current char into al
     ! cmp al, 65          ; if (al < 65) then
     ! jb NoRot            ;    move to next character
     ! cmp al, 122         ; if (al > 122) then
     ! ja NoRot            ;    move to next character
     ! cmp al, 96          ; if (al > 96 and < 123) then
     ! ja Rot13Lower       ;    lower case letter
     ! cmp al, 64          ; if (al > 64 and < 91) then
     ! ja Rot13Upper       ;    upper case letter
     ! jmp NoRot           ; else move to next character
 
  Rot13Lower:
     ! add al, 13          ; Rotate 13 steps forward
     ! cmp al, 122         ; if al became > 122 then
     ! ja AdjChar          ;    wrap around by subtracting 26
     ! mov [esi], al       ; else write changed char back into a$
     ! jmp NoRot           ; move to next character
 
  Rot13Upper:
     ! add al, 13          ; Rotate 13 steps forward
     ! cmp al, 90          ; if al became > 90 then
     ! ja AdjChar          ;    wrap around by subtracting 26
     ! mov [esi], al       ; else write changed char back into a$
     ! jmp NoRot           ; move to next character
 
  AdjChar:
     ! sub al, 26          ; wrap around by subtracting 26 (13 x 2)
     ! mov [esi], al       ; and write changed char back into a$
 
  NoRot:
     ! inc esi             ; move to next character
     ! dec ecx             ; decrease length counter (ecx)
     ! jnz BgnRot13Loop    ; jump to begin of loop if not zero
 
END SUB
 
'
' Main entrance
'--------------------------------------------------------------------
FUNCTION PBMAIN () AS LONG
  LOCAL hDlg AS DWORD
  DIALOG NEW 0, "PB/WIN asm ANSI String handling routines",,, 307, 95, _
                %WS_CAPTION OR %WS_SYSMENU, 0 TO hDlg
 
  CONTROL ADD BUTTON,  hDlg, %ID_UPPER,   "m&UCASER",     174,  14,  60,  14
  CONTROL ADD BUTTON,  hDlg, %ID_LOWER,   "m&LCASER",     174,  28,  60,  14
  CONTROL ADD BUTTON,  hDlg, %ID_MCASE,   "m&MCASER",     174,  42,  60,  14
  CONTROL ADD BUTTON,  hDlg, %ID_TCASE,   "m&TCASER",     174,  62,  60,  14
  CONTROL ADD BUTTON,  hDlg, %ID_ROT13,   "m&ROT13",      174,  76,  60,  14
 
  CONTROL ADD BUTTON,  hDlg, %ID_PBUCASE, "PB's UCASE",   242,  14,  60,  14
  CONTROL ADD BUTTON,  hDlg, %ID_PBLCASE, "PB's LCASE",   242,  28,  60,  14
  CONTROL ADD BUTTON,  hDlg, %ID_PBMCASE, "PB's MCASE",   242,  42,  60,  14
 
  CONTROL ADD CHECKBOX, hDlg, %ID_CHECK, "100,000 times ", 242, 60,  65,  14
 
  CONTROL ADD BUTTON,  hDlg, %IDCANCEL,   "E&xit",        242, 76,  60,  14
 
  CONTROL ADD LABEL,   hDlg, -1,       "Original text",     6,   5,  60,  9
  CONTROL ADD TEXTBOX, hDlg, %ID_TEXT,  $TXT,               5,  14, 160, 30, _
                       %ES_MULTILINE OR %WS_VSCROLL OR %ES_AUTOVSCROLL OR _
                       %ES_WANTRETURN, %WS_EX_CLIENTEDGE
 
  CONTROL ADD LABEL,   hDlg, -1,       "Result", 6,  50,  60,  9
  CONTROL ADD TEXTBOX, hDlg, %ID_TEXT2, "",      5,  59, 160, 30, _
                       %ES_MULTILINE OR %WS_VSCROLL OR %ES_AUTOVSCROLL OR _
                       %ES_WANTRETURN, %WS_EX_CLIENTEDGE
 
  DIALOG SHOW MODAL hDlg CALL DlgProc
 
END FUNCTION
 
'
' Main callback
'--------------------------------------------------------------------
CALLBACK FUNCTION DlgProc() AS LONG
  SELECT CASE CBMSG
      CASE %WM_INITDIALOG
         CALL initAnsiTables 'initalize global ANSI character tables
 
     CASE %WM_COMMAND      ' <- a control is calling
        IF CBCTLMSG = %BN_CLICKED OR CBCTLMSG = 1 THEN
           SELECT CASE CBCTL
              CASE %ID_UPPER TO %ID_PBMCASE
                 CALL CaseProc(CBHNDL, CBCTL)    '<- actions
 
              CASE %IDCANCEL : DIALOG END CBHNDL '<- End program
            END SELECT
            EXIT FUNCTION
         END IF
 
  END SELECT
END FUNCTION
 
'
' Command central
'--------------------------------------------------------------------
SUB CaseProc(BYVAL hDlg AS DWORD, BYVAL cmd AS LONG)
  LOCAL c AS LONG, Ln AS LONG, lChk AS LONG, _
        txt AS STRING, txt2 AS STRING, t AS SINGLE
 
  CONTROL GET TEXT hDlg, %ID_TEXT TO txt
  CONTROL GET CHECK hDlg, %ID_CHECK TO lChk
 
  IF lChk = 0 THEN 'Do once..
     SELECT CASE cmd
        CASE %ID_LOWER   : txt2 = mLCASER(txt)
        CASE %ID_MCASE   : txt2 = mMCASER(txt)
        CASE %ID_TCASE   : txt2 = mTCASER(txt)
        CASE %ID_UPPER   : txt2 = mUCASER(txt)
        CASE %ID_ROT13   : txt2 = mROT13(txt)  ' ROT13 encryption/decryption
        CASE %ID_PBLCASE : txt2 = LCASE$(txt)  ' PB's Lower case..
        CASE %ID_PBMCASE : txt2 = MCASE$(txt)  ' PB's Mixed case..
        CASE %ID_PBUCASE : txt2 = UCASE$(txt)  ' PB's Upper case - for compare
     END SELECT
     CONTROL SET TEXT hDlg, %ID_TEXT2, txt2
 
  ELSE  'Repeat 100,000 times
     MOUSEPTR 11
     t = TIMER
     FOR c = 1 TO 100000
        Ln = Ln + LEN(txt)
        SELECT CASE cmd
           CASE %ID_LOWER   : txt2 = mLCASER(txt)
           CASE %ID_MCASE   : txt2 = mMCASER(txt)
           CASE %ID_TCASE   : txt2 = mTCASER(txt)
           CASE %ID_UPPER   : txt2 = mUCASER(txt)
           CASE %ID_ROT13   : txt2 = mROT13(txt)  ' ROT13 encryption/decryption
           CASE %ID_PBLCASE : txt2 = LCASE$(txt)  ' PB's Lower case..
           CASE %ID_PBMCASE : txt2 = MCASE$(txt)  ' PB's Mixed case..
           CASE %ID_PBUCASE : txt2 = UCASE$(txt)  ' PB's Upper case - for compare
        END SELECT
     NEXT
 
     t = TIMER - t
     MOUSEPTR 1
     CONTROL SET TEXT hDlg, %ID_TEXT2, txt2
     MSGBOX "Repeated:           100,000 times." & $CRLF & _
            "Processed:      " & USING$("#,###", Ln) & "  bytes." & $CRLF & _
            "Elapsed time:  "  & FORMAT$(t, "0.0000") & "  seconds.", _
            %MB_ICONINFORMATION, "Information"
  END IF
 
END SUB
